package gov.usgs.earthquake.nshmp.netcdf.reader;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;

import gov.usgs.earthquake.nshmp.data.XySequence;
import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.netcdf.data.BoundingData;
import gov.usgs.earthquake.nshmp.netcdf.data.StaticData;

public class GroundMotionsNetcdfUtils {

  /**
   * Check whether bounding ground motions contain the same: Site classes, IMTs
   * per each site class, and ground motions.
   *
   * @param a static data A
   * @param b static B
   */
  static void checkBoundingGroundMotion(
      StaticData<XySequence> a,
      StaticData<XySequence> b) {
    checkState(a.size() == b.size(), "Maps are not the same size");
    checkState(a.keySet().containsAll(b.keySet()), "Site classes do not match");
    a.keySet().forEach(key -> NetcdfUtils.checkXySequence(a.get(key), b.get(key)));
  }

  /**
   * Checks bounding hazard maps contain the same: Site classes, IMTs per each
   * site class, and ground motions per each IMT
   *
   * @param boundingData The bounding ground motions
   */
  static void checkBoundingGroundMotions(
      BoundingData<XySequence> boundingData,
      Location location) {
    checkArgument(boundingData.containsKey(location), "Location not in bounding hazards");
    boundingData.keySet().stream()
        .filter(loc -> loc.equals(location))
        .forEach(key -> {
          checkBoundingGroundMotion(boundingData.get(location), boundingData.get(key));
        });
  }

  static StaticData<XySequence> cleanData(StaticData<XySequence> data) {
    var cleanData = StaticData.<XySequence> builder();

    data.entrySet()
        .forEach(entry -> {
          cleanData.put(entry.getKey(),
              NetcdfUtils.cleanData(entry.getValue(), NetcdfUtils.GROUND_MOTION_PRECISION));
        });

    return cleanData.build();
  }

  /*
   * Linear interpolation of data values to a target point
   */
  static StaticData<XySequence> linearInterpolateGroundMotions(
      StaticData<XySequence> v1,
      StaticData<XySequence> v2,
      double frac) {
    checkBoundingGroundMotion(v1, v2);

    var staticData = StaticData.<XySequence> builder();

    v1.keySet().forEach(siteClass -> {
      var v1Data = v1.get(siteClass).yValues().toArray();
      var v2Data = v2.get(siteClass).yValues().toArray();
      var target = new double[v1Data.length];

      for (int i = 0; i < v1Data.length; i++) {
        target[i] = v1Data[i] * (1 - frac) + v2Data[i] * frac;
      }

      var xValues = v1.get(siteClass).xValues().toArray();
      staticData.put(siteClass, XySequence.create(xValues, target));
    });

    return cleanData(staticData.build());
  }
}
