package gov.usgs.earthquake.nshmp.netcdf.www;

import java.util.List;
import java.util.stream.Collectors;

import gov.usgs.earthquake.nshmp.data.XySequence;
import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.netcdf.NetcdfGroundMotions;
import gov.usgs.earthquake.nshmp.netcdf.NetcdfVersion;
import gov.usgs.earthquake.nshmp.netcdf.data.StaticData;
import gov.usgs.earthquake.nshmp.netcdf.www.NetcdfWsUtils.Key;
import gov.usgs.earthquake.nshmp.netcdf.www.Query.Service;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ResponseMetadata;
import gov.usgs.earthquake.nshmp.www.WsUtils;

import io.micronaut.http.HttpRequest;

/**
 * Handles service calls for ground motions.
 *
 * @see NetcdfController
 *
 * @author U.S. Geological Survey
 */
public class NetcdfServiceGroundMotions extends NetcdfService<Query> {

  static final String SERVICE_DESCRIPTION = "Get static ground motions from a NetCDF file";
  static final String SERVICE_NAME = "AASHTO-2023 Web Service";
  static final String X_LABEL = "Period (s)";
  static final String Y_LABEL = "Spectral Acceleration (g)";

  public NetcdfServiceGroundMotions(NetcdfGroundMotions netcdf) {
    super(netcdf);
  }

  @Override
  ResponseBody<String, Metadata> getMetadataResponse(HttpRequest<?> request) {
    var metadata = new Metadata(request, SERVICE_DESCRIPTION);

    return ResponseBody.<String, Metadata> usage()
        .metadata(new ResponseMetadata(NetcdfVersion.appVersions()))
        .name(SERVICE_NAME)
        .request(NetcdfWsUtils.getRequestUrl(request))
        .response(metadata)
        .url(NetcdfWsUtils.getRequestUrl(request))
        .build();
  }

  @Override
  String getServiceName() {
    return SERVICE_NAME;
  }

  @Override
  SourceModel getSourceModel() {
    return new SourceModel();
  }

  @Override
  NetcdfGroundMotions netcdf() {
    return (NetcdfGroundMotions) netcdf;
  }

  @Override
  ResponseBody<?, ?> processRequest(HttpRequest<?> httpRequest, Query query, Service service) {
    var site = Location.create(query.longitude, query.latitude);
    var requestData = new RequestData(site);
    var url = NetcdfWsUtils.getRequestUrl(httpRequest);

    switch (service) {
      case CURVES:
        return processCurves(requestData, url);
      case CURVES_BY_SITE_CLASS:
        requestData = new RequestDataSiteClass(site, query.siteClass);
        return processCurvesSiteClass(
            (RequestDataSiteClass) requestData, url);
      default:
        throw new RuntimeException("Netcdf service [" + service + "] not found");
    }
  }

  @Override
  ResponseBody<RequestDataSiteClass, ResponseData<ServiceResponseMetadata>> processCurvesSiteClass(
      RequestDataSiteClass request,
      String url) {
    WsUtils.checkValue(Key.LATITUDE, request.latitude);
    WsUtils.checkValue(Key.LONGITUDE, request.longitude);
    WsUtils.checkValue(Key.SITE_CLASS, request.siteClass);
    var curves = netcdf().staticData(request.site, request.siteClass);
    var responseData = toResponseData(request, curves);

    return ResponseBody.<RequestDataSiteClass, ResponseData<ServiceResponseMetadata>> success()
        .metadata(new ResponseMetadata(NetcdfVersion.appVersions()))
        .name(SERVICE_NAME)
        .request(request)
        .response(responseData)
        .url(url)
        .build();
  }

  @Override
  ResponseBody<RequestData, List<ResponseData<ServiceResponseMetadata>>> processCurves(
      RequestData request,
      String url) {
    WsUtils.checkValue(Key.LATITUDE, request.latitude);
    WsUtils.checkValue(Key.LONGITUDE, request.longitude);
    var curves = netcdf().staticData(request.site);
    var responseData = toList(request.site, curves);

    return ResponseBody.<RequestData, List<ResponseData<ServiceResponseMetadata>>> success()
        .metadata(new ResponseMetadata(NetcdfVersion.appVersions()))
        .name(SERVICE_NAME)
        .request(request)
        .response(responseData)
        .url(url)
        .build();
  }

  List<ResponseData<ServiceResponseMetadata>> toList(
      Location site,
      StaticData<XySequence> curves) {
    return curves.entrySet().stream()
        .map(entry -> {
          var request = new RequestDataSiteClass(site, entry.getKey());
          return toResponseData(request, entry.getValue());
        })
        .collect(Collectors.toList());
  }

  ResponseData<ServiceResponseMetadata> toResponseData(
      RequestDataSiteClass request,
      XySequence curves) {
    var metadata = new ServiceResponseMetadata(
        request.siteClass,
        X_LABEL,
        Y_LABEL);
    return new ResponseData<>(metadata, curves);
  }
}
