package gov.usgs.earthquake.nshmp.netcdf.reader;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;

import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.netcdf.data.NetcdfDataHazardCurves;
import gov.usgs.earthquake.nshmp.netcdf.data.NetcdfShape;
import gov.usgs.earthquake.nshmp.netcdf.data.NetcdfShape.IndexKey;
import gov.usgs.earthquake.nshmp.netcdf.reader.NetcdfUtils.Key;

import ucar.ma2.DataType;
import ucar.ma2.InvalidRangeException;
import ucar.nc2.Group;
import ucar.nc2.Variable;

/**
 * Read in hazard curve NetCDF files.
 *
 * @author U.S. Geological Survey
 */
public class ReaderHazardCurves extends Reader {

  public ReaderHazardCurves(Group targetGroup) throws IOException {
    super(targetGroup);
  }

  @Override
  public NetcdfDataHazardCurves netcdfData() {
    return (NetcdfDataHazardCurves) netcdfData;
  }

  @Override
  NetcdfDataHazardCurves readData(Group targetGroup) throws IOException {
    var coords = super.readData(targetGroup);
    var vImls = targetGroup.findVariableLocal(Key.IMLS);

    // get map of IMLs
    var imls = mapImls(vImls, coords.imts());

    return NetcdfDataHazardCurves.builder()
        .imls(imls)
        .imts(coords.imts())
        .latitudes(coords.latitudes())
        .longitudes(coords.longitudes())
        .scienceBaseMetadata(coords.scienceBaseMetadata())
        .siteClasses(coords.siteClasses())
        .vs30Map(coords.vs30Map())
        .build();
  }

  @Override
  NetcdfShape buildNetcdfShape(Group group) {
    var vData = group.findVariableLocal(Key.HAZARD);
    return NetcdfShape.builder()
        .add(IndexKey.IML, vData.findDimensionIndex(Key.IMLS))
        .add(IndexKey.IMT, vData.findDimensionIndex(Key.IMT))
        .add(IndexKey.LATITUDE, vData.findDimensionIndex(Key.LAT))
        .add(IndexKey.LONGITUDE, vData.findDimensionIndex(Key.LON))
        .add(IndexKey.SITE_CLASS, vData.findDimensionIndex(Key.SITE_CLASS))
        .build();
  }

  /*
   * convert 2D Iml variable (dimensions Imt, Iml) to map of Imls by Imt
   *
   * TODO: use MultiMap or SetMultiMap (etc.) to store unique IML sets? Could
   * then also initialize the underlying XySequence objects for reading in the
   * hazard curves...
   */
  private Map<Imt, double[]> mapImls(Variable vImls, List<Imt> imts) {
    var nIml = vImls.getShape(1);
    Map<Imt, double[]> imtImlMap = new EnumMap<Imt, double[]>(Imt.class);

    for (int j = 0; j < imts.size(); j++) {
      var imt = imts.get(j);

      // set origin and shape of double[] Imls to read
      var origin = new int[] { j, 0 };
      var shape = new int[] { 1, nIml };

      try {
        imtImlMap.put(
            imt,
            (double[]) vImls.read(origin, shape).reduce().get1DJavaArray(DataType.DOUBLE));
      } catch (IOException | InvalidRangeException e) {
        var msg = "Failed read attempt for vImls with origin: " +
            Arrays.toString(origin) + ", shape: " + Arrays.toString(shape);
        throw new RuntimeException(msg);
      }
    }

    return Collections.unmodifiableMap(imtImlMap);
  }
}
