package gov.usgs.earthquake.nshmp.netcdf.data;

import static com.google.common.base.Preconditions.checkState;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import gov.usgs.earthquake.nshmp.Maths;
import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.gmm.NehrpSiteClass;
import gov.usgs.earthquake.nshmp.netcdf.reader.NetcdfUtils;

/**
 * Data info from the NetCDF file.
 *
 * @author U.S. Geological Survey
 */
public class NetcdfData {
  private final Location minimumBounds;
  private final Location maximumBounds;
  private final List<Imt> imts;
  private final double[] latitudes;
  private final double[] longitudes;
  private final ScienceBaseMetadata scienceBaseMetadata;
  private final List<NehrpSiteClass> siteClasses;
  private final Map<NehrpSiteClass, Double> vs30Map;

  protected NetcdfData(Builder builder) {
    var minLatitude = Arrays.stream(builder.latitudes).min().getAsDouble();
    var maxLatitude = Arrays.stream(builder.latitudes).max().getAsDouble();
    var minLongitude = Arrays.stream(builder.longitudes).min().getAsDouble();
    var maxLongitude = Arrays.stream(builder.longitudes).max().getAsDouble();

    minimumBounds = Location.create(
        Maths.round(minLongitude, NetcdfUtils.LOCATION_PRECISION),
        Maths.round(minLatitude, NetcdfUtils.LOCATION_PRECISION));
    maximumBounds = Location.create(
        Maths.round(maxLongitude, NetcdfUtils.LOCATION_PRECISION),
        Maths.round(maxLatitude, NetcdfUtils.LOCATION_PRECISION));
    imts = builder.imts;
    latitudes = builder.latitudes;
    longitudes = builder.longitudes;
    scienceBaseMetadata = builder.scienceBaseMetadata;
    siteClasses = builder.siteClasses;
    vs30Map = builder.vs30Map;
  }

  /**
   * Returns the maximum location bounds
   */
  public Location maximumBounds() {
    return maximumBounds;
  }

  /**
   * Returns the minimum location bounds
   */
  public Location minimumBounds() {
    return minimumBounds;
  }

  /**
   * Return the Imts
   */
  public List<Imt> imts() {
    return List.copyOf(imts);
  }

  /**
   * Returns the latitudes.
   */
  public double[] latitudes() {
    return latitudes.clone();
  }

  /**
   * Returns the longitudes.
   */
  public double[] longitudes() {
    return longitudes.clone();
  }

  /**
   * Returns the science base info
   */
  public ScienceBaseMetadata scienceBaseMetadata() {
    return scienceBaseMetadata;
  }

  /**
   * Return the site classes
   */
  public List<NehrpSiteClass> siteClasses() {
    return List.copyOf(siteClasses);
  }

  /**
   * Returns the VS30 map
   */
  public Map<NehrpSiteClass, Double> vs30Map() {
    return new TreeMap<>(vs30Map);
  }

  public static Builder builder() {
    return new Builder();
  }

  public static class Builder {
    List<Imt> imts;
    double[] latitudes;
    double[] longitudes;
    List<NehrpSiteClass> siteClasses;
    ScienceBaseMetadata scienceBaseMetadata;
    Map<NehrpSiteClass, Double> vs30Map;

    Builder() {}

    public Builder imts(List<Imt> imts) {
      this.imts = imts;
      return this;
    }

    public Builder latitudes(double[] latitudes) {
      this.latitudes = latitudes;
      return this;
    }

    public Builder longitudes(double[] longitudes) {
      this.longitudes = longitudes;
      return this;
    }

    public Builder scienceBaseMetadata(ScienceBaseMetadata scienceBaseMetadata) {
      this.scienceBaseMetadata = scienceBaseMetadata;
      return this;
    }

    public Builder siteClasses(List<NehrpSiteClass> siteClasses) {
      this.siteClasses = siteClasses;
      return this;
    }

    public Builder vs30Map(Map<NehrpSiteClass, Double> vs30Map) {
      this.vs30Map = vs30Map;
      return this;
    }

    public NetcdfData build() {
      checkBuildState();
      return new NetcdfData(this);
    }

    void checkBuildState() {
      checkState(!imts.isEmpty(), "Must add imts");
      checkState(!(latitudes.length == 0), "Must add latitude");
      checkState(!(longitudes.length == 0), "Must add longitudes");
      checkState(scienceBaseMetadata != null, "Must set science base metadata");
      checkState(!siteClasses.isEmpty(), "Must add site classes");
      checkState(!vs30Map.isEmpty(), "Must add vs30s");
    }
  }

}
